use git2::{Branch, Repository};

use crate::common::{self, Refractr};
use crate::config::{ConfigFile, Config};
use std::fs;
use std::env;
use std::path::{Path, PathBuf};
use git2::{Branches, Error};

fn set_up_work_dir(work_dir: PathBuf) -> String {
  if let Err(e) = fs::create_dir_all(&work_dir) {
    panic!("refractr: could not create working directory: {}", work_dir.to_string_lossy())
  }
  work_dir.to_string_lossy().to_string()
}

fn fast_forward(repo_dir: &str, branches: &Option<Vec<String>>) -> Result<(), Error> {
  let repo = Repository::open(repo_dir)?;
  let branch_list: Vec<String> = match branches {
    Some(repo_branches) => repo_branches.to_vec(),
    None => {
      let mut strings = Vec::new();
      let remote_branches = repo.branches(Some(git2::BranchType::Remote))?;
      for branch in remote_branches {
        if let Ok((b, _)) = branch {
          if let Ok(Some(name)) = b.name() {
            strings.push(name.to_string());
          }
        }
      }
      strings
    }
  };

  repo.find_remote("origin")?.fetch(&branch_list, None, None)?;
  Ok(())
}

pub fn start(refractr: Refractr, cfgs: Vec<ConfigFile>) -> std::io::Result<()> {
  common::verbose(refractr.verbose, 3, format!("Starting main refractr loop"));
  for i in &cfgs {
    // set up the working directory
    common::verbose(refractr.verbose, 3, format!("Loaded config: {}", i.path));
    let path_str = set_up_work_dir(match &i.config.work_dir {
      None => {
        if cfg!(windows) {
          PathBuf::from(format!("\"{}\\refractr\"", match env::var("TEMP") {
            Ok(val) => val,
            Err(_) => format!("This shouldn't happen!")
          }))
        } else {
          PathBuf::from("/tmp/refractr")
        }
      },
      Some(path) => PathBuf::from(path)
    });

    common::verbose(refractr.verbose, 2, format!("Created working directory: {}", &path_str));
    let repo_name = match &i.config.from.split("/").last() {
      Some(split) => split.to_string(),
      None => panic!("refractr: failed to parse repository name")
    };

    // make initial clone
    common::verbose(refractr.verbose, 1, format!("Cloning repository: {}", &i.config.from));
    let repo_dir = &format!("{}/{}", &path_str, repo_name);
    match Repository::clone(&i.config.from, Path::new(&repo_dir)) {
      Ok(repo) => (),
      Err(e) => {
        eprintln!("refractr: warning: found existing repo at {}, attempting to use", repo_dir);
        if let Err(e) = fast_forward(repo_dir, &i.config.branches) {
          eprintln!("failed");
        }
      }
    }
  }

  Ok(())
}

